import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/device_managment/ceiling_sensor/bloc/bloc.dart';
import 'package:syncrow_web/pages/device_managment/ceiling_sensor/bloc/event.dart';
import 'package:syncrow_web/pages/device_managment/ceiling_sensor/bloc/state.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_display_data.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_static_widget.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_status.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_update_data.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/helpers/responsice_layout_helper/responsive_layout_helper.dart';

class CeilingSensorControls extends StatelessWidget
    with HelperResponsiveLayout {
  const CeilingSensorControls({super.key, required this.device});

  final AllDevicesModel device;

  @override
  Widget build(BuildContext context) {
    final isLarge = isLargeScreenSize(context);
    final isMedium = isMediumScreenSize(context);
    return BlocProvider(
      create: (context) => CeilingSensorBloc(deviceId: device.uuid ?? '')
        ..add(CeilingInitialEvent()),
      child: BlocBuilder<CeilingSensorBloc, CeilingSensorState>(
        builder: (context, state) {
          if (state is CeilingLoadingInitialState) {
            return const Center(child: CircularProgressIndicator());
          } else if (state is CeilingUpdateState) {
            return GridView(
              padding: const EdgeInsets.symmetric(horizontal: 50),
              shrinkWrap: true,
              physics: const NeverScrollableScrollPhysics(),
              gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                crossAxisCount: isLarge
                    ? 3
                    : isMedium
                        ? 2
                        : 1,
                mainAxisExtent: 133,
                crossAxisSpacing: 12,
                mainAxisSpacing: 12,
              ),
              children: [
                PresenceState(
                  value: state.ceilingSensorModel.presenceState,
                ),
                PresenceDisplayValue(
                  value: state.ceilingSensorModel.sportsPara.toString(),
                  postfix: '',
                  description: 'Sports para',
                ),
                PresenceDisplayValue(
                  value: state.ceilingSensorModel.presenceRange.toString(),
                  postfix: 'm',
                  description: 'Detection Range',
                ),
                PresenceUpdateData(
                  value: state.ceilingSensorModel.sensitivity.toDouble(),
                  title: 'Sensitivity:',
                  minValue: 1,
                  maxValue: 5,
                  steps: 1,
                  action: (int value) {
                    context.read<CeilingSensorBloc>().add(
                          CeilingChangeValueEvent(
                            code: 'sensitivity',
                            value: value,
                          ),
                        );
                  },
                ),
                PresenceUpdateData(
                  value: state.ceilingSensorModel.maxDistance.toDouble(),
                  title: 'Maximum Distance:',
                  minValue: 0,
                  maxValue: 500,
                  steps: 50,
                  description: 'm',
                  action: (int value) => context.read<CeilingSensorBloc>().add(
                        CeilingChangeValueEvent(
                          code: 'moving_max_dis',
                          value: value,
                        ),
                      ),
                ),
                PresenceUpdateData(
                    value:
                        (state.ceilingSensorModel.noBodyTime.toDouble() / 3600)
                            .roundToDouble(),
                    title: 'Nobody Time:',
                    minValue: 0,
                    maxValue: 300000,
                    steps: 5000,
                    description: 'hr',
                    action: (int value) => context
                        .read<CeilingSensorBloc>()
                        .add(CeilingChangeValueEvent(
                          code: 'none_body_time',
                          value: value,
                        ))),
                GestureDetector(
                  onTap: () {},
                  child: const PresenceStaticWidget(
                    icon: Assets.illuminanceRecordIcon,
                    description: 'Presence Record',
                  ),
                ),
                GestureDetector(
                  onTap: () {},
                  child: const PresenceStaticWidget(
                    icon: Assets.helpDescriptionIcon,
                    description: 'Help Description',
                  ),
                ),
              ],
            );
          } else {
            return const Center(child: Text('Error fetching status'));
          }
        },
      ),
    );
  }
}
